@@ -10,7 +10,7 @@ import os
 
 
 class CustomDataset(Dataset):
-    def __init__(self, root_dir, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = False, resample = None):
+    def __init__(self, root_dir, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = True, resample = None):
         self.root_dir = root_dir
         self.image_name = []
         self.info_name = []
