@@ -14,8 +14,8 @@ from ingradient_library.sampling import *
 
 
 class Trainer(object):
-    def __init__(self, tr_dataloader, model,  optimizer, scheduler, losses, regulizers = [], is_deep_supervision = False,
-                deep_supervision_weight = 0.5, n_epoch = 1000, val_dataloader = None, save_path = None):
+    def __init__(self, tr_dataloader, model,  optimizer, losses, regulizers = [], is_deep_supervision = False,
+                deep_supervision_weight = 0.5, n_epoch = 1000, val_dataloader = None, save_path = None, scheduler = None):
  
         self.model = model
         self.optimizer = optimizer
@@ -65,13 +65,10 @@ class Trainer(object):
 
         for i in range(len(self.losses)):
             loss = self.losses[i] #여러 Loss들에 대해
-            print("output, target", output.shape, target.shape)
             temp = loss(output, target) * torch.tensor(self.regulizers[i]).float().to(output.device.index)
-            print('before', temp.shape)
             for _ in range(len(temp.shape)-1):
                 temp = temp.mean(-1)
             writer.add_scalar('Iteration\Loss_'+ str(type(loss)), temp[0], epoch) #(batch, -1) 로 나오게끔 resize
-            print('after', temp.shape)
             if self.is_deep_supervision == True:
                 temp = temp.view(batch_size, deep_supervision_size, -1)
             if i == 0:
@@ -85,7 +82,7 @@ class Trainer(object):
         else:
             result = losses.mean()
 
-        return result
+        return result.contiguous()
     
 
 
@@ -115,7 +112,8 @@ class Trainer(object):
                     def closure():
                         self.optimizer.zero_grad()
                         output = self.model(images)
-                        loss = self.calculate_loss(output, seg, writer, e)
+                        current_iter = (self.current_index* self.tr_dl.num_iteration + self.tr_dl.now_iter) /(self.tr_dl.num_iteration*len(self.tr_dl.dataset))
+                        loss = self.calculate_loss(output, seg, writer, e + current_iter)
                         loss.backward()
                         return loss
                     self.optimizer.step(closure)
@@ -123,21 +121,22 @@ class Trainer(object):
                 else:
                     output = self.model(images)
                     self.optimizer.zero_grad()
-                    print(self.tr_dl.current_index)
-                    loss = self.calculate_loss(output, seg, writer, e)
+                    current_iter = (self.current_index* self.tr_dl.num_iteration + self.tr_dl.now_iter) /(self.tr_dl.num_iteration*len(self.tr_dl.dataset))
+                    loss = self.calculate_loss(output, seg, writer, e + current_iter)
                     loss.backward()
                     self.optimizer.step()
                 
-                if not self.is_deep_supervision:
-                    plt.imshow(torch.argmax(output[0,:,output.shape[2]//2], dim = 0).detach().cpu(), vmax =1 , vmin= 0)
+                if self.tr_dl.current_index < 10 and not isinstance(self.optimizer, SAMSGD):
+                    if not self.is_deep_supervision:
+                        plt.imshow(torch.argmax(output[0,:,output.shape[2]//2], dim = 0).detach().cpu(), vmax =1 , vmin= 0)
+                        plt.show()
+                    elif isinstance(self.optimizer, SAMSGD):
+                        plt.imshow(torch.argmax(output[0,0,:,output.shape[2]//2], dim = 0).detach().cpu(), vmax =1 , vmin= 0)
+                        plt.show()
+                    plt.imshow(images[0,0,output.shape[2]//2].detach().cpu())
                     plt.show()
-                else:
-                    plt.imshow(torch.argmax(output[0,0,:,output.shape[2]//2], dim = 0).detach().cpu(), vmax =1 , vmin= 0)
+                    plt.imshow(seg[0,output.shape[2]//2].detach().cpu(), vmax =1 , vmin= 0)
                     plt.show()
-                plt.imshow(images[0,0,output.shape[2]//2].detach().cpu())
-                plt.show()
-                plt.imshow(seg[0,output.shape[2]//2].detach().cpu(), vmax =1 , vmin= 0)
-                plt.show()
 
 
             if self.val_dl != None:
@@ -150,12 +149,14 @@ class Trainer(object):
                     with torch.no_grad():
                         images, seg = self.val_dl.generate_train_batch()
                         output = self.model(images)
-                        loss = self.calculate_loss(output, seg, writer, e)
+                        current_iter = (self.current_index* self.tr_dl.num_iteration + self.tr_dl.now_iter) /(self.tr_dl.num_iteration*len(self.tr_dl.dataset))
+                        loss = self.calculate_loss(output, seg, writer, e + current_iter)
                         if self.visualization != None:
                             if self.val_dl.current_index == 0:
                                 self.visualization(output, seg)
-                
-            self.scheduler.step()
+
+            if self.scheduler != None: 
+                self.scheduler.step()
             if self.save_path != None:
                 file_name = 'epoch'+ str(e) + '_model_state_dict.pkl'
                 torch.save(self.model.state_dict(), os.path.join(self.save_path, file_name))
